@@ -1,12 +1,12 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
-use std::os::unix::io::RawFd;
+use std::os::unix::io::OwnedFd;
 
+use adw::prelude::*;
 use adw::subclass::prelude::*;
-use adw::traits::BreakpointBinExt;
 use ashpd::desktop::camera;
 use gettextrs::gettext;
+use gtk::CompositeTemplate;
 use gtk::{gio, glib};
-use gtk::{prelude::*, CompositeTemplate};
 
 use super::CameraControls;
 use crate::{config, utils};
@@ -23,6 +23,7 @@ mod imp {
         pub provider: OnceCell<aperture::DeviceProvider>,
         pub players: RefCell<Option<gtk::MediaFile>>,
         settings: OnceCell<gio::Settings>,
+        pub permission_denied: Cell<bool>,
 
         pub recording_duration: Cell<u32>,
         pub recording_source: RefCell<Option<glib::source::SourceId>>,
@@ -133,7 +134,7 @@ mod imp {
                         let camera = selected_item.downcast::<aperture::Camera>().ok();
 
                         if matches!(obj.imp().viewfinder.state(), aperture::ViewfinderState::Ready | aperture::ViewfinderState::Error) {
-                            obj.imp().viewfinder.set_camera(camera);
+                            obj.set_camera_inner(camera);
                         }
                     }
                 }),
@@ -210,8 +211,7 @@ impl Camera {
     pub async fn start(&self) {
         let provider = self.imp().provider.get().unwrap();
 
-        let ctx = glib::MainContext::default();
-        ctx.spawn_local(
+        glib::spawn_future_local(
             glib::clone!(@weak self as obj, @strong provider => async move {
                 match stream().await {
                     Ok(fd) => {
@@ -219,9 +219,20 @@ impl Camera {
                             log::error!("Could not use the camera portal: {err}");
                         };
                     }
+                    Err(ashpd::Error::Portal(ashpd::PortalError::NotAllowed(err))) => {
+                        // We don't start the device provider if we are not
+                        // allowed to use cameras.
+                        log::warn!("Permission to use the camera portal denied: {err}");
+                        obj.imp().permission_denied.set(true);
+                        obj.update_state();
+                        return;
+                    },
                     Err(err) => log::warn!("Could not use the camera portal: {err}"),
                 }
-                if let Err(err) = provider.start() {
+                if let Err(err) = provider.start_with_default(glib::clone!(@weak obj => @default-return false, move |camera| {
+                    let stored_id = obj.imp().settings().string("last-camera-id");
+                    !stored_id.is_empty() && id_from_pw(camera) == stored_id
+                })) {
                     log::error!("Could not start the device provider: {err}");
                 } else {
                     log::debug!("Device provider started");
@@ -284,10 +295,21 @@ impl Camera {
             pos += 1;
         };
         if let Some(camera) = provider.camera(pos) {
-            self.imp().viewfinder.set_camera(Some(camera));
+            self.set_camera_inner(Some(camera));
         }
     }
 
+    fn set_camera_inner(&self, camera: Option<aperture::Camera>) {
+        let imp = self.imp();
+
+        if let Some(ref camera) = camera {
+            let id = id_from_pw(camera);
+            imp.settings().set_string("last-camera-id", &id).unwrap();
+        }
+
+        imp.viewfinder.set_camera(camera);
+    }
+
     fn play_shutter_sound(&self) {
         // If we don't hold a reference to it there is a condition race which
         // will cause the sound to play only sometimes.
@@ -299,13 +321,16 @@ impl Camera {
     }
 
     fn active_controls(&self) -> &CameraControls {
+        let imp = self.imp();
+        let bp1 = &*imp.single_landscape_bp;
+        let bp2 = &*imp.dual_landscape_bp;
         if self
             .current_breakpoint()
-            .is_some_and(|bp| bp.to_string().contains("max-aspect-ratio: 4/3"))
+            .is_some_and(|bp| &bp == bp1 || &bp == bp2)
         {
-            &self.imp().camera_controls_horizontal
+            &imp.camera_controls_horizontal
         } else {
-            &self.imp().camera_controls_vertical
+            &imp.camera_controls_vertical
         }
     }
 
@@ -372,6 +397,14 @@ impl Camera {
         imp.camera_controls_vertical.set_gallery(&gallery);
     }
 
+    pub fn stop_stream(&self) {
+        self.imp().viewfinder.stop_stream();
+    }
+
+    pub fn start_stream(&self) {
+        self.imp().viewfinder.start_stream();
+    }
+
     pub fn toggle_guidelines(&self) {
         let imp = self.imp();
 
@@ -393,7 +426,8 @@ impl Camera {
         let camera = imp.viewfinder.camera();
         if let Some(pos) = imp
             .selection
-            // gtk::SingleSelection will Always returns glib::Object as its gio::ListModel::item_type().
+            // gtk::SingleSelection will Always returns glib::Object as its
+            // gio::ListModel::item_type().
             .iter::<glib::Object>()
             .enumerate()
             .find(|(_pos, cam)| {
@@ -408,6 +442,13 @@ impl Camera {
 
     fn update_state(&self) {
         let imp = self.imp();
+
+        if imp.permission_denied.get() {
+            imp.spinner.stop();
+            imp.stack.set_visible_child_name("not-found");
+            return;
+        }
+
         match imp.viewfinder.state() {
             aperture::ViewfinderState::Loading => {
                 imp.spinner.start();
@@ -482,9 +523,14 @@ impl Camera {
     }
 }
 
-async fn stream() -> ashpd::Result<RawFd> {
+async fn stream() -> ashpd::Result<OwnedFd> {
     let proxy = camera::Camera::new().await?;
     proxy.request_access().await?;
 
     proxy.open_pipe_wire_remote().await
 }
+
+// Id used to identify the last-used camera.
+fn id_from_pw(camera: &aperture::Camera) -> glib::GString {
+    camera.display_name()
+}
